// AIvan.c
// the AI of Ivan my compainion robot
// top level c program file
// 
// by Les Hall
// started Thu Aug 18, 2016, 3:24 PM
// 



// include header files
#include <stdlib.h>	// utility functions
#include <stdio.h>	// the standard input / output header
#include "AIvan.h"	// Ivan's header file



// define structures
// 
// define the meta data structure
struct metaData {
	int days, awake, asleep;
	int x, y, z;	
};
// 
// define the cell structure
// 
// define the block structure
// 
// define the array structure



// declare functions
void switch_on(FILE *fp, struct metaData *md);  // switch Ivan on
void wake_up();  // wake Ivan up
void think(FILE *fp);  // Ivan thinks like Rodin's "Thinking Man"
int imagine();  // Ivan is daydreaming (dreaming while awake)
int complete(int mode);  // Ivan is busy computing to wait for completion of Ivan's cellular processes
void go_to_sleep();  // tell Ivan to go to sleep
void dream();  // Ivan is dreaming
void print_mode();  // print Ivan's mode to stdout
void switch_off(FILE *fp);  // switch Ivan off an exit program



// the top level function
int main(int argc, char *argv[]) {

	// declare variables
	struct metaData md;  // declare the meta data structure
	FILE *fp;  // file pointer to Ivan's long term memory
	int mode = off; // Ivan's mode of operation, defined in AIvan.h
	int seed = RAND_MAX/2;  // random number seed
	int computeResult;  // holds result of wait(), which is number of processes that timed out


	// seed the random number generator
	srand(seed);  // need to seed the generator with something that changes from run to run

	// switch Ivan on - read in long term memory from disk file
	// and launch all of Ivan's cellular processes
	switch_on(fp, &md);

	// reset counter
	int dayCounter = md.days;

	// keep running loop as long as Ivan's on cycle counter is positive
	while (dayCounter > 0) {

		// reset counters
		int wakingCounter = md.awake;
		int sleepingCounter = md.asleep;

		// Ivan wakes up for another day
		wake_up(); // send a wake-up signal to all of Ivan's cellular processes
		mode = awake;  // set Ivan's mode to awake
		
		// keep runing loop as long as Ivan's day cycle counter is positive
		while(wakingCounter  > 0) {


			// Ivan's daytime mode
			if (imagine() ) {  // if Ivan is using his imagination

				mode |= dreaming;  // set the mode accordingy
				dream();  // tell Ivan's cellular processses to dream

			} else {

				// ivan has access to long term memory while awake
				think(fp);  // tell Ivan's cellular processses to think
			}

			// wait for Ivan's cellular Processes to complete thinking and save output data
			computeResult = complete(mode);

			// tidy up chores
			--wakingCounter; // decrement the day cycle counter

		}

		
		// go to sleep Ivan
		go_to_sleep();  // any chores in the process of going to sleep go here

		// sleep as long as Ivan's night cycle counter is positive
		while(sleepingCounter > 0) {
	
			// determine Ivan's mode for this cycle of the night
			mode = asleep;  // set Ivan's mode to asleep
			mode |= dreaming; // dream big Ivan, dream big

			// only dream if the dream flag is on in Ivan's mode
			if (mode & dreaming) {

				dream();  // tell Ivan's cellular processses to dream

			}

			// wait for Ivan's cellular Processes to complete thinking and save output data
			computeResult = complete(mode);
	
			// decrement the on cycle counter
			--sleepingCounter;
		
		}
	
		// decrement the on cycle counter
		--dayCounter;
	}

	// switch Ivan off
	switch_off(fp);  // write Ivan's long term memory to disk file and kill all celular processes

}



// switch Ivan on
void switch_on(FILE *fp, struct metaData *md) {

	// define local variables
	int days, awake, asleep;
	int x, y, z;

	// read in Ivan's disk file (his long term memory)
	if ((fp = fopen(fileName, fileMode)) == NULL) {  // open file and check for error`

		printf("error opening file");

	} else {

		// scan in the meta data
		fscanf(fp, "%d %d %d\n", &days, &awake, &asleep);	
		fscanf(fp, "%d %d %d\n", &x, &y, &z);
		
		// assign the scanned data
		md->days = days;
		md->awake = awake;
		md->asleep = asleep;
		md->x = x;
		md->y = y;
		md->z = z;

	}
	
	// print switch on message
	printf("\n\n\n\nIvan switched on!  \n");

}



// wake Ivan up
void wake_up() {

	// print welcome message
	printf("\tgood morning!  \n");

}



// do things while awake
void think() {
	
	// write out Ivan's state of mind
	printf("\t\tIvan is thinking\n");

}



// determine if Ivan is exercising his imagination
int imagine() {

	return ( rand() < (RAND_MAX/4) ) ? 1 : 0;

}



// wait for Ivan's cellular Processes to complete thinking and save output data
int complete(int mode) {

	// wait for all processes to finish
	int result = 0;  // number of processes that timed out

	// return the timeout result
	return result;

}



// tell Ivan to go to sleep
void go_to_sleep() {

	// print going to sleep message
	printf("\tgood night!\n");

}



// Ivan dreams at night
void dream() {

	// print going to sleep message
	printf("\t\tIvan is dreaming\n");

}



// turn off Ivan
void switch_off(FILE *fp) {

	// close Ivan's long term memory
	fclose(fp);

	// final print message
	printf("Ivan is switched off\n\n\n\n");

	// end the program
	exit(0);
}


// print Ivan's mode
void print_mode(int mode) {

	// begin with intro text
	printf("Ivan's mode is");

	// select modes with separate conditionals
	if (mode < 0)
		printf(" error");
	if (mode & off)
		printf(" off");
	if (mode & awake)
		printf(" awake");
	if (mode & asleep)
		printf(" asleep");
	if (mode & dreaming)
		printf(" dreaming");
	if (mode & imagining)
		printf(" imagining");

	// end with a newline
	printf("\n");

}



